# Notes

- [Notes](#notes)
  - [模块std::fmt](#模块std::fmt)
    - [用法](#用法)
    - [位置参数](#位置参数)
    - [命名参数](#命名参数)
    - [宽度](#宽度)
    - [填充/对齐](#填充/对齐数)

## 模块std::fmt
- 用于格式化和打印String的实用程序。
- 该模块包含对format!语法扩展的运行时支持。该宏在编译器中实现以发出对该模块的调用，以便在运行时将参数格式化为字符串。

### 用法
```
format!("Hello");                 // => "Hello"
format!("Hello, {}!", "world");   // => "Hello, world!"
format!("The number is {}", 1);   // => "The number is 1"
format!("{:?}", (3, 4));          // => "(3, 4)"
format!("{value}", value=4);      // => "4"
format!("{} {}", 1, 2);           // => "1 2"
format!("{:04}", 42);             // => "0042" with leading zeros
format!("{:#?}", (100, 200));     // => "(
                                  //       100,
                                  //       200,
                                  //     )"
```
### 位置参数
每个格式化参数都可以指定它引用的值参数，如果省略，则假定它是“下一个参数”。例如，格式字符串{} {} {}将采用三个参数，并且它们将按照给定的顺序进行格式化。{2} {1} {0}然而，格式字符串 会以相反的顺序格式化参数。
```
format!( "{1} {} {0} {}" , 1 , 2 ); // => "2 1 1 2"
```
### 命名参数
命名参数列在参数列表的末尾，语法如下：
```
identifier '=' expression
```
例子如下：
```
format!("{name} {}", 1, name = 2);          // => "2 1"
format!("{a} {c} {b}", a="a", b='b', c=3);  // => "a 3 b"
```
### 宽度
这是格式应采用的“最小宽度”的参数。如果值的字符串没有填充这么多字符，则填充/对齐指定的填充将用于占用所需的空间。
：后面是最小宽度，$前面表示最小宽度实参。
使用美元语法引用参数不会影响“下一个参数”计数器，因此按位置引用参数或使用命名参数通常是个好主意。
```
// All of these print "Hello x!"
println!("Hello {:5}!", "x");
println!("Hello {:1$}!", "x", 5);
println!("Hello {1:0$}!", 5, "x");
println!("Hello {:width$}!", "x", width = 5);
```
### 填充/对齐
```
assert_eq!(format!("Hello {:<5}!", "x"),  "Hello x    !");
assert_eq!(format!("Hello {:-<5}!", "x"), "Hello x----!");
assert_eq!(format!("Hello {:^5}!", "x"),  "Hello   x  !");
assert_eq!(format!("Hello {:>5}!", "x"),  "Hello     x!");
Run

```
可选的填充字符和对齐方式通常与width参数一起提供 。它必须在 之前width、之后定义:。这表明如果被格式化的值小于 width一些额外的字符将被打印在它周围。针对不同的对齐方式，填充有以下变体：

[fill]<- 参数在width列中左对齐
[fill]^- 参数在width列中居中对齐
[fill]>- 参数在width列中右对齐
非数字的默认填充/对齐方式是空格并左对齐。数字格式化程序的默认值也是一个空格字符，但右对齐。如果0为数字指定了标志（见下文），则隐式填充字符为 0.

请注意，某些类型可能无法实现对齐。特别是，它通常不会为Debugtrait实现。确保应用填充的一个好方法是格式化您的输入，然后填充这个结果字符串以获得您的输出：
```
println!("Hello {:^15}!", format!("{:?}", Some("hi"))); // => "Hello   Some("hi")   !"
Run

```
