# Notes

- [Notes](#notes)
  - [泛型](#泛型)
    - [枚举泛型](#枚举泛型)
    - [结构体泛型](#结构体泛型)

  - [特征 trait](#特征 trait)
    - [trait bound](#trait bound)
    - [类似继承](#类似继承)
    - [Move closure](#Move closure)

## 泛型
### 枚举泛型
```
enum Option<T> {
    Some(T),
    None,
}
```
### 结构体泛型
```
struct Point<T>{
    x:T,
    y:T,
}  //字段 x 和 y 都是 相同类型的

struct PointTU<T, U>{
    x:T,
    y:U,
}  //字段  x 和 y 可以有不同类型

fn main() {
    let int = Point{x:4, y:4};
    let fl = Point{x:4.4, y:5.4};

    println!("{}, {}", int.x, fl.y);

    let hun = PointTU{x:"44", y:4.5};
    println!("{}, {}", hun.x, hun.y);

}
```
## 特征 trait
trait用来实现多态。类似C++中的接口,可以重载。
```
pub trait Summary{
    fn summarize(&self)->String;
}

pub struct NewsArticle{
    pub headline:String,
    pub location:String,
    pub author:String,
    pub content:String,
}

pub struct Tweet{
    pub username:String,
    pub content:String,
    pub reply:bool,
    pub retweet:bool,
}

impl Summary for NewsArticle{
  fn summarize(&self)->String{
      format!("{}, by {} ({})", self.headline, self.author, self.location)
  }
}

impl Summary for Tweet {
    fn summarize(&self) -> String {
        return format!("{}: {}", self.username, self.content);
    }
}

fn main() {
   let tweet = Tweet{
       username:String::from("horse_ebooks"),
       content:String::from("I have nothing"),
       reply:false,
       retweet:false,
   };

   println!("{}", tweet.summarize());

    let article = NewsArticle {
    headline: String::from("NewsArticle_headline"),
    location: String::from("Pittsburgh, PA, USA"),
    author: String::from("Iceburgh"),
    content: String::from("The Pittsburgh Penguins once again are the best hockey team in the NHL."),
   };

    println!("{}", article.summarize());
}
```
### trait bound
```
//写法一
fn print_ information<T: GetName+GetAge>(item:T){ 

}
//写法二
fn print_ information<T> (item: T) 
    where T: GetName+GetAge{

}
```
### 类似继承
```
//实现trait
trait GatName{
    fn get_name(&self)->&String;
}
trait PrintName{
    fn print_name(&self);
}

//实现了GatName的就可以实现PrintName
impl<T :GatName> PrintName for T{
    fn print_name(&self){
        println!("name ={}",self.get_name());
    }
}

struct Student{
    name :String,
}
impl GatName for Student{
    fn get_name(&self)->&String{
        &(self.name)
    }
}
fn main() {
    let s =Student{ name:String::from("Leedia")};
    s.print_name();
    println!("Hello,world!");
}
```
### Move closure
使用move关键字，强制closure获得所有权，但下面的例子得注意一下：
```    
let num = 5;
let owns_num = move |x: i32| x + num;
```

